<?php

$title = "GT 4.0 Developer Notes: Java WS Core";

include_once( "/mcs/www-unix.globus.org/include/globus_header.inc" ); 
?>



<p class="small"><a href="http://www.globus.org">Globus</a> &gt; <a href="http://www-unix.globus.org/toolkit/">Toolkit</a> &gt; <a href="http://www-unix.globus.org/toolkit/docs/">Documents</a> &gt; <a href="http://www-unix.globus.org/toolkit/docs/development/index.html">Development</a> &gt;  <a href="http://www-unix.globus.org/toolkit/docs/development/3.9.2/index.html">3.9.2</a> &lt; </p>
<h1>GT 4.0 Developer Notes: Java WS Core</h1>

<h2>Tomcat</h2>

<p>
To deploy GT4 installation into Tomcat run:
<pre>
cd $GLOBUS_LOCATION
ant -f share/globus_wsrf_common/tomcat/tomcat.xml deployTomcat -Dtomcat.dir=<b>&lt;tomcat.dir&gt</b>
</pre>

Where <i>&lt;tomcat.dir&gt;</i> is an <b>absolute</b> path to the Tomcat installation directory. 
</p>

<p>
To create a <code>.war</code> of GT4 installation do:
<pre>
cd $GLOBUS_LOCATION
ant -f share/globus_wsrf_common/tomcat/tomcat.xml war -Dwar.file=<b>&lt;war.file&gt</b>
</pre>

Where <i>&lt;war.file&gt;</i> specifies an <b>absolute</b> path of the war file. <br><br>Please note that deploying a <code>war</code> file might not be enough to have a working GT4 deployment. For example, in some cases the <code>xalan.jar</code> must be placed in the <code>endorsed</code> directory of the container.
</p>


<h2>Testing</h2>

<h3>Conventions</h3>

<p>
Group your tests under <i>PackageTests.java</i> and/or <i>SecurityTests.java</i> (The naming is important because these names are used to discover the tests dynamically). Put all tests that require any type of credentials in <i>SecurityTests.java</i> test suite. 
</p>

<h3>Running</h3>

<p>
To execute the tests on GT4 install run (assuming the tests have been deployed into GT4 install):

<pre>
cd $GLOBUS_LOCATION
ant -f share/globus_wsrf_test/runtests.xml runServer -Dtests.jar=&lt;test.jar&gt;
</pre>

Where <i>&lt;test.jar&gt;</i> is an <b>absolute</b> path to the jar file that contains the tests. By default, the tests that use a container will try to access a container running at <i>http://localhost:8080/wsrf/services/</i>. To specify a different container use <i>-Dtest.server.url=&lt;url&gt;</i> property. To execute PackageTests only specify <i>-DbasicTestsOnly=true</i>. To execute SecurityTests only specify <i>-DsecurityTestsOnly=true</i>. <br>
The test reports will be put in <i>$GLOBUS_LOCATION/share/globus_wsrf_test/test-reports</i> directory by default. The different test reports directory can be specified by passing <i>-Djunit.reports.dir=&lt;directory&gt;</i>.
</p>

<h2>Activation</h2>

<h3>Service</h3>

<h4>Axis RPCProvider</h4>

<p>
Instance of a service is always created on an invocation. It depends on the 'scope' setting if one instance of the service is used for all invocations (<i>Application</i> scope), if one instance is created per invocation (<i>Request</i> scope), or one instance per session (<i>Session</i> scope). The <i>Request</i> scope is default (if scope parameter is not set in the deployment descriptor). If service implements the <a href="http://java.sun.com/j2ee/1.4/docs/api/javax/xml/rpc/server/ServiceLifecycle.html">javax.xml.rpc.server.ServiceLifecycle</a> interface the lifecycle methods they will be called accordingly to the scope setting as a service instance is created and destroyed. For example, in <i>Application</i> scope <i>destroy()</i> will be called on container shutdown, and in <i>Request</i> scope it will be called after the service method is called. JAAS credentials will not be associated with a thread when a service instance is created.
</p>

<h4>Globus RPCProvider</h4>

<p>
Only <i>Application</i> and <i>Request</i> scopes are supported. <br>
If <i>Request</i> scope things behave just like in Axis RPCProvider (service/providers instances are created per invocation, no JAAS credentials, ServiceLifecycle methods called right before and after service method invocation). <br>
If <i>Application</i> scope service/provider instances are created either on first invocation or on container startup if <i>"loadOnStartup"</i> parameter in the deployment descriptor is set:

<pre>
&lt;parameter name="loadOnStartup" value="true'/&gt;
</pre>

This will work in the same way in the standalone container and in Tomcat.
If the service or the container is configured with a security descriptor, the appropriate credentials will be associated with the thread during activation (using JAAS). Also, during activation a basic Axis MessageContext will be associated with the thread with only <i>Constants.MC_HOME_DIR</i>, <i>Constants.MC_CONFIGPATH</i>, and the right target service properties set (see <a href="#MessageContext">MessageContext Properties</a> section for details). If service or providers implement the <a href="http://java.sun.com/j2ee/1.4/docs/api/javax/xml/rpc/server/ServiceLifecycle.html">javax.xml.rpc.server.ServiceLifecycle</a> interface the lifecycle methods will be called accordingly.
</p>

<h3>ResourceHome</h3>

<p>
ResourceHome will be activated on the first service invocation or during container startup if <i>"loadOnStartup"</i> parameter is set. If ResourceHome get activated in such ways, proper MessageContext and/or JAAS subject will be associated with the thread. Also proper MessageContext and/or JAAS subject will be associated with the thread if the ResourceHome gets activated by a separate thread directly looking up the ResourceHome implementation in JNDI. This actually applies to all <i>resource</i> entries directly defined under <i>service</i> entries in <i>jndi-config.xml</i> and configured with <i>org.globus.wsrf.jndi.BeanFactory</i> factory.
</p>

<h4>ServiceResourceHome</h4>

<p>
If you are using ServiceResourceHome please make sure to deploy the service with <i>"loadOnStartup"</i> option enabled and in <i>Application</i> scope. That will ensure that the ResourceHome is initialized with the right service/resource.
</p>

<a name="MessageContext"></a>
<h2>MessageContext Properties</h2>

<ul>
<li>
Constants.MC_HOME_DIR - the base directory from which the wsdl files are loaded from. 
</li>
<li>
Constants.MC_CONFIGPATH - the base directory from which different configuration files are loaded from. For example, use that to load files from etc/ directory.
</li>
<li>
Constants.MC_REMOTE_ADDR - contains the IP address of the client
</li>
<li>
MessageContext.TRANS_URL - the request url
</li>
</ul>

<p>
In the standalone container the <i>Constants.MC_HOME_DIR</i> and <i>Constants.MC_CONFIGPATH</i> properties will usually point to the same directory. However, in Tomcat they will point to two different directories so make sure to use the right property to load the configuration files.
</p>

<h2>Local Invocations</h2>

<p>
Services in the container can be invoked locally. Local invocations work just like remote invocations (all handlers are called, messages get serialized/deserialized) but messages do not travel over the network - everything happens in memory. 
</p>
<p>
Local invocations can only be made on the server side and only when a Axis MessageContext is associated with a current thread. URLs with "<i>local</i>" protocol name are used for local invocations. To invoke a service locally do:

<table cellpadding="5">

<tr>
<td valign="top">1</td>
<td valign="top">
<p>Register "<i>local</i>" protocol handler:</p>
<pre>
import org.globus.axis.transport.local.LocalTransportUtils;
...
<b>LocalTransportUtils.init();</b>
...
</pre>
</td>
</tr>

<tr>
<td valign="top">2</td>
<td valign="top">
<p>Create a service URL with "<i>local</i>" protocol:</p>
<pre>
URL url = new URL(<b>"local:///wsrf/services/MyService"</b>);
</pre>
</td>
</tr>

<tr>
<td valign="top">3</td>
<td valign="top">
<p>Configure the service stub for local invocation and make the call:</p>
<pre>
MyServiceAddressingLocator locator = 
       new MyServiceAddressingLocator();
MyService port = locator.getMyServicePort(url);

<b>LocalTransportUtils.enableLocalTransport((Stub)port);</b>

port.hello();
</pre>

</table>
</p>

<h3>Tomcat</h3>

<p>
Special configuration is required to enable local invocations in Tomcat. The "<i>local</i>" protocol URL handler must be installed in a different way. 
[ADD MORE INFO HERE HOW].
</p>



<h2>Using ResourceHomeImpl</h2>

<p>
If <i>ResourceHomeImpl</i> is configured with resource class that implements the <i>PersistentResource</i> interface it will store the resource objects wrapped in Java <a href="http://java.sun.com/j2se/1.4.2/docs/api/java/lang/ref/SoftReference.html">SoftReference</a>. That allows the JVM to automatically reclaim these resource objects thus reducing the memory usage. Since the JVM can decide to reclaim these objects at any point sometimes a resource object can be reclaimed between two subsequent invocations on the same resource. This for example can cause the state of the resource to be reloaded from disk on each call.<br>
To prevent the JVM from reclaiming the resource objects so quickly a cache can be setup up to hold direct references to these objects. We provide a basic LRU (least recently used) cache implementation. Other cache implementations can be used as long as they implement the <i>org.globus.wsrf.utils.cache.Cache</i> interface.
</p>

<p>
To configure a cache for <i>ResourceHomeImpl</i> first define a cache resource:
</p>
<pre>
&lt;resource name="cache" 
             type="org.globus.wsrf.utils.cache.LRUCache"&gt;
  &lt;resourceParams&gt;
     &lt;parameter&gt;
        &lt;name&gt;factory&lt;/name&gt;
        &lt;value&gt;org.globus.wsrf.jndi.BeanFactory&lt;/value&gt;
     &lt;/parameter&gt;
     &lt;parameter&gt;
        &lt;name&gt;timeout&lt;/name&gt;
        &lt;value&gt;120000&lt;/value&gt;
     &lt;/parameter&gt;
  &lt;/resourceParams&gt;
&lt;/resource>
</pre>
<p>
In this case a LRU cache is configured. The "<i>timeout</i>" parameter (in ms) is used to specify the idle time of the resource object before it is removed from cache.
<br>
The same cache resource can be reused in different services but usually once cache per service will be configured. 
</p>
<p>
Once the cache resource is defined add the "<i>cacheLocation</i>" parameter to the service <i>home</i> resource. The "<i>cacheLocation</i>" parameter value is the JNDI name of the cache resource:
</p>
<pre>
&lt;service name="CounterService"&gt;
   &lt;resource name="home" type="..."&gt;
      &lt;resourceParams&gt;
         ...<b>
         &lt;parameter&gt;
            &lt;name&gt;cacheLocation&lt;/name&gt;
            &lt;value&gt;java:comp/env/CounterService/cache&lt;/value&gt;
         &lt;/parameter&gt;</b>
         ...
      &lt;/resourceParams&gt;
   ...
</pre>
<p>
Please note that once the object is removed from the cache it is still up to the JVM to actually reclaim the object.
</p>

<h2>Proxy Support</h2>

<p>
A basic proxy support is provided. A <b><code>org.globus.wsrf.proxy.port</code></b> system property can be set to the port of the proxy server (the proxy server must run on the same machine as the container). This will make any code that uses <code>ServiceHost</code> or <code>AddressingUtils</code> API return the address of the proxy server instead of the container. This could be useful for example for debugging purposes.<br>
The <b><code>org.globus.wsrf.proxy.port</code></b> system property can be passed to <code>globus-start-container</code> script via <code>GLOBUS_OPTIONS</code> environment property. For example:

<pre>
setenv GLOBUS_OPTIONS="-Dorg.globus.wsrf.proxy.port=5555"
globus-start-container
</pre>

Please note that not all of the core code will obey the proxy port setting.
 
</p>

<?php include("http://www-unix.globus.org/include/globus_footer.inc"); ?>
